{-# OPTIONS_GHC -freduction-depth=400 #-}

{-|
Module      : Gargantext.API.Swagger
Description : Swagger API generation
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

---------------------------------------------------------------------
module Gargantext.API.Swagger (
    backendApiProxy
  , openApiDoc
  ) where

---------------------------------------------------------------------
import Control.Lens ((?~))
import Data.OpenApi (OpenApi)
import Data.Swagger
import Data.Version (showVersion)
import Gargantext.API.Routes.Named qualified as Named
import Gargantext.Orphans () -- instance HasOpenApi
import Gargantext.Prelude
import Gargantext.Prelude
import qualified Paths_gargantext as PG -- cabal magic build module
import Servant
import Servant.OpenApi (toOpenApi)
import Servant.Swagger

backendApiProxy :: Proxy (ToServantApi Named.BackEndAPI)
backendApiProxy = Proxy

-- | OpenAPI Specification. It supports programmatically adding the `Bearer` to the `Authorization`
-- header, which makes the Swagger UI actually useable, unlike the 'swaggerDoc' counterpart which
-- is stuck on version 2.x of the OpenAPI spec.
openApiDoc :: OpenApi
openApiDoc = toOpenApi backendApiProxy

-- | Swagger Specifications
_swaggerDoc :: Swagger
_swaggerDoc = toSwagger backendApiProxy
  & info.title       .~ "GarganText"
  & info.version     .~ (cs $ showVersion PG.version)
  -- & info.base_url     ?~ (URL "http://gargantext.org/")
  & info.description ?~ "REST API specifications"
  -- & tags             .~ Set.fromList [Tag "Garg" (Just "Main perations") Nothing]
  & applyTagsFor (subOperations backendApiProxy backendApiProxy)
                 ["Gargantext" & description ?~ "Main operations"]
  & info.license     ?~ ("AGPLV3 (English) and CECILL (French)" & url ?~ URL urlLicence )
    where
        urlLicence = "https://gitlab.iscpif.fr/gargantext/haskell-gargantext/blob/master/LICENSE"
